"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importStar(require("express"));
const serialize_error_1 = require("serialize-error");
exports.DEFAULT_ENDPOINT = '/';
const DEFAULT_HANDLER = (_req, res, next) => {
    try {
        res.status(204).send();
    }
    catch (e) {
        console.error('Error while processing request: ', e);
        next(e);
    }
};
const DEFAULT_ERROR_HANDLER = (err, _req, res, _next) => {
    if (err) {
        res.status(500).json(serialize_error_1.serializeError(err));
    }
};
/**
 *
 * @param {RequestHandlerParams[]} [appMiddleware=[]]
 * @param {RequestHandlerParams[]} [routeMiddleware=[]]
 * @param {ExpressEndpointAssertor[]} [transformedRequestAssertors=[]]
 * @param transformedResponseAssertors
 * @param {ErrorAssertor[]} [errorAssertors=[]]
 * @param {RequestHandlerParams[]} [routerMiddleware=[]]
 * @param errorHandler
 * @param {RequestHandlerParams} [handler] - if handler is not specified, by default sends 204 NO CONTENT
 * @param {string | EndpointDefinition} [endpoint='/'] - if endpoint is passed as string, or not specified, by default GET method is used.
 */
function newExpressApp({ appMiddleware = [], routeMiddleware = [], routerMiddleware = [], transformedRequestAssertors = [], transformedResponseAssertors = [], errorAssertors = [], errorHandler = DEFAULT_ERROR_HANDLER, handler = DEFAULT_HANDLER, endpoint = exports.DEFAULT_ENDPOINT }) {
    const assertorErrorHandler = (err, _req, _res, next) => {
        try {
            errorAssertors.forEach(assertor => {
                assertor(err);
            });
        }
        catch (e) {
            return next(e);
        }
        return next(err);
    };
    const app = express_1.default();
    // Resolve endpoint from params
    let method;
    let path;
    if (typeof endpoint === 'string') {
        method = 'get';
        path = endpoint;
    }
    else {
        method = endpoint.method.toLowerCase();
        path = endpoint.path;
    }
    const router = express_1.Router();
    routerMiddleware.forEach(middleware => {
        router.use(middleware);
    });
    const assertorRequestMiddleware = transformedRequestAssertors.map(assertor => {
        return (req, _res, next) => __awaiter(this, void 0, void 0, function* () {
            yield assertor(req);
            next();
        });
    });
    const assertorResponseMiddleware = transformedResponseAssertors.map(assertor => {
        return (_req, res, next) => __awaiter(this, void 0, void 0, function* () {
            yield assertor(res);
            next();
        });
    });
    // @ts-ignore
    router[method](path, routeMiddleware, assertorRequestMiddleware, assertorResponseMiddleware, handler);
    appMiddleware.forEach(middleware => {
        app.use(middleware);
    });
    if (errorAssertors && errorAssertors.length > 0) {
        app.use((_req, _res, next) => {
            return next(new Error(`Expected error to be thrown, but it wasn't`));
        });
    }
    app.use(router);
    app.use(assertorErrorHandler);
    app.use(errorHandler);
    return app;
}
exports.newExpressApp = newExpressApp;
//# sourceMappingURL=expressApp.js.map