"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const koa_router_1 = __importDefault(require("koa-router"));
exports.DEFAULT_ENDPOINT = '/';
const DEFAULT_HANDLER = (ctx, next) => {
    ctx.status = 204;
    next();
};
/**
 *
 * @param {Middleware[]} [appMiddleware=[]]
 * @param {Middleware[]} [routerMiddleware=[]]
 * @param {KoaEndpointAssertor[]} [transformedRequestAssertors=[]]
 * @param {Middleware} [handler] - if handler is not specified, by default sends 204 NO CONTENT
 * @param {string | EndpointDefinition} [endpoint='/'] - if endpoint is passed as string, or not specified, by default GET method is used.
 */
function newKoaApp({ appMiddleware = [], routerMiddleware = [], transformedRequestAssertors = [], handler = DEFAULT_HANDLER, endpoint = exports.DEFAULT_ENDPOINT }) {
    const app = new koa_1.default();
    appMiddleware.forEach((middlewareEntry) => {
        app.use(middlewareEntry);
    });
    const router = new koa_router_1.default();
    routerMiddleware.forEach((middlewareEntry) => {
        router.use(middlewareEntry);
    });
    transformedRequestAssertors.forEach(assertor => {
        router.use((ctx, next) => __awaiter(this, void 0, void 0, function* () {
            yield assertor(ctx);
            next();
        }));
    });
    // Resolve endpoint from params
    let method;
    let path;
    if (typeof endpoint === 'string') {
        method = 'get';
        path = endpoint;
    }
    else {
        method = endpoint.method.toLowerCase();
        path = endpoint.path;
    }
    // @ts-ignore
    router[method](path, handler);
    app.use(router.routes()).use(router.allowedMethods());
    return app;
}
exports.newKoaApp = newKoaApp;
//# sourceMappingURL=koaApp.js.map